from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GeneratedScorer")


@_attrs_define
class GeneratedScorer:
    """
    Attributes:
        id (Union[None, Unset, str]):
        name (Union[None, Unset, str]):
    """

    id: Union[None, Unset, str] = UNSET
    name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id: Union[None, Unset, str]
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        name: Union[None, Unset, str]
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        generated_scorer = cls(id=id, name=name)

        generated_scorer.additional_properties = d
        return generated_scorer

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import IntEnum


class XRayClass(IntEnum):
    VALUE_0 = 0
    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5
    VALUE_6 = 6
    VALUE_7 = 7
    VALUE_8 = 8
    VALUE_9 = 9
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12
    VALUE_13 = 13
    VALUE_14 = 14
    VALUE_15 = 15
    VALUE_16 = 16
    VALUE_17 = 17
    VALUE_18 = 18
    VALUE_19 = 19
    VALUE_20 = 20
    VALUE_21 = 21
    VALUE_22 = 22
    VALUE_23 = 23
    VALUE_24 = 24
    VALUE_25 = 25
    VALUE_26 = 26
    VALUE_27 = 27
    VALUE_28 = 28
    VALUE_29 = 29
    VALUE_30 = 30
    VALUE_31 = 31
    VALUE_32 = 32
    VALUE_33 = 33
    VALUE_34 = 34
    VALUE_35 = 35
    VALUE_36 = 36
    VALUE_37 = 37
    VALUE_38 = 38
    VALUE_39 = 39
    VALUE_40 = 40
    VALUE_41 = 41
    VALUE_42 = 42
    VALUE_43 = 43
    VALUE_44 = 44
    VALUE_45 = 45
    VALUE_46 = 46
    VALUE_47 = 47
    VALUE_48 = 48
    VALUE_49 = 49
    VALUE_50 = 50
    VALUE_51 = 51
    VALUE_52 = 52
    VALUE_53 = 53

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ExecutionStatus(str, Enum):
    ERROR = "error"
    FAILED = "failed"
    NOT_TRIGGERED = "not_triggered"
    PAUSED = "paused"
    TIMEOUT = "timeout"
    TRIGGERED = "triggered"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Span")


@_attrs_define
class Span:
    """
    Attributes:
        id (int):
        is_active (bool):
        pred (str):
        span_end (int):
        span_start (int):
        confidence (Union[None, Unset, float]):
        data_error_potential (Union[None, Unset, float]):
        error_type (Union[None, Unset, str]):
        gold (Union[None, Unset, str]):
        is_drifted (Union[None, Unset, bool]):
        is_on_the_boundary (Union[None, Unset, bool]):
        x (Union[None, Unset, float]):
        y (Union[None, Unset, float]):
    """

    id: int
    is_active: bool
    pred: str
    span_end: int
    span_start: int
    confidence: Union[None, Unset, float] = UNSET
    data_error_potential: Union[None, Unset, float] = UNSET
    error_type: Union[None, Unset, str] = UNSET
    gold: Union[None, Unset, str] = UNSET
    is_drifted: Union[None, Unset, bool] = UNSET
    is_on_the_boundary: Union[None, Unset, bool] = UNSET
    x: Union[None, Unset, float] = UNSET
    y: Union[None, Unset, float] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        is_active = self.is_active

        pred = self.pred

        span_end = self.span_end

        span_start = self.span_start

        confidence: Union[None, Unset, float]
        if isinstance(self.confidence, Unset):
            confidence = UNSET
        else:
            confidence = self.confidence

        data_error_potential: Union[None, Unset, float]
        if isinstance(self.data_error_potential, Unset):
            data_error_potential = UNSET
        else:
            data_error_potential = self.data_error_potential

        error_type: Union[None, Unset, str]
        if isinstance(self.error_type, Unset):
            error_type = UNSET
        else:
            error_type = self.error_type

        gold: Union[None, Unset, str]
        if isinstance(self.gold, Unset):
            gold = UNSET
        else:
            gold = self.gold

        is_drifted: Union[None, Unset, bool]
        if isinstance(self.is_drifted, Unset):
            is_drifted = UNSET
        else:
            is_drifted = self.is_drifted

        is_on_the_boundary: Union[None, Unset, bool]
        if isinstance(self.is_on_the_boundary, Unset):
            is_on_the_boundary = UNSET
        else:
            is_on_the_boundary = self.is_on_the_boundary

        x: Union[None, Unset, float]
        if isinstance(self.x, Unset):
            x = UNSET
        else:
            x = self.x

        y: Union[None, Unset, float]
        if isinstance(self.y, Unset):
            y = UNSET
        else:
            y = self.y

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {"id": id, "is_active": is_active, "pred": pred, "span_end": span_end, "span_start": span_start}
        )
        if confidence is not UNSET:
            field_dict["confidence"] = confidence
        if data_error_potential is not UNSET:
            field_dict["data_error_potential"] = data_error_potential
        if error_type is not UNSET:
            field_dict["error_type"] = error_type
        if gold is not UNSET:
            field_dict["gold"] = gold
        if is_drifted is not UNSET:
            field_dict["is_drifted"] = is_drifted
        if is_on_the_boundary is not UNSET:
            field_dict["is_on_the_boundary"] = is_on_the_boundary
        if x is not UNSET:
            field_dict["x"] = x
        if y is not UNSET:
            field_dict["y"] = y

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        is_active = d.pop("is_active")

        pred = d.pop("pred")

        span_end = d.pop("span_end")

        span_start = d.pop("span_start")

        def _parse_confidence(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        confidence = _parse_confidence(d.pop("confidence", UNSET))

        def _parse_data_error_potential(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        data_error_potential = _parse_data_error_potential(d.pop("data_error_potential", UNSET))

        def _parse_error_type(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        error_type = _parse_error_type(d.pop("error_type", UNSET))

        def _parse_gold(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        gold = _parse_gold(d.pop("gold", UNSET))

        def _parse_is_drifted(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        is_drifted = _parse_is_drifted(d.pop("is_drifted", UNSET))

        def _parse_is_on_the_boundary(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        is_on_the_boundary = _parse_is_on_the_boundary(d.pop("is_on_the_boundary", UNSET))

        def _parse_x(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        x = _parse_x(d.pop("x", UNSET))

        def _parse_y(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        y = _parse_y(d.pop("y", UNSET))

        span = cls(
            id=id,
            is_active=is_active,
            pred=pred,
            span_end=span_end,
            span_start=span_start,
            confidence=confidence,
            data_error_potential=data_error_potential,
            error_type=error_type,
            gold=gold,
            is_drifted=is_drifted,
            is_on_the_boundary=is_on_the_boundary,
            x=x,
            y=y,
        )

        span.additional_properties = d
        return span

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.metric_critique_column import MetricCritiqueColumn


T = TypeVar("T", bound="MetricCritiqueColumns")


@_attrs_define
class MetricCritiqueColumns:
    """
    Attributes:
        metric_critique_columns (list['MetricCritiqueColumn']):
    """

    metric_critique_columns: list["MetricCritiqueColumn"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        metric_critique_columns = []
        for metric_critique_columns_item_data in self.metric_critique_columns:
            metric_critique_columns_item = metric_critique_columns_item_data.to_dict()
            metric_critique_columns.append(metric_critique_columns_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"metric_critique_columns": metric_critique_columns})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric_critique_column import MetricCritiqueColumn

        d = src_dict.copy()
        metric_critique_columns = []
        _metric_critique_columns = d.pop("metric_critique_columns")
        for metric_critique_columns_item_data in _metric_critique_columns:
            metric_critique_columns_item = MetricCritiqueColumn.from_dict(metric_critique_columns_item_data)

            metric_critique_columns.append(metric_critique_columns_item)

        metric_critique_columns = cls(metric_critique_columns=metric_critique_columns)

        metric_critique_columns.additional_properties = d
        return metric_critique_columns

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.create_user_response import CreateUserResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.social_login_request import SocialLoginRequest
from ...types import UNSET, Response, Unset


def _get_kwargs(*, body: SocialLoginRequest, signup_token: Union[None, Unset, str] = UNSET) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    json_signup_token: Union[None, Unset, str]
    if isinstance(signup_token, Unset):
        json_signup_token = UNSET
    else:
        json_signup_token = signup_token
    params["signup_token"] = json_signup_token

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {"method": "post", "url": "/users/social", "params": params}

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[CreateUserResponse, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = CreateUserResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[CreateUserResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: SocialLoginRequest,
    signup_token: Union[None, Unset, str] = UNSET,
) -> Response[Union[CreateUserResponse, HTTPValidationError]]:
    """Create Or Verify User Social

     Create a user using a social login provider.

    All social users are created with `email_is_verified=True`, don't need to be invited and are by
    default read-only
    (unless they are the first user, in which case they are set to admin).

    Args:
        signup_token (Union[None, Unset, str]):
        body (SocialLoginRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreateUserResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(body=body, signup_token=signup_token)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    body: SocialLoginRequest,
    signup_token: Union[None, Unset, str] = UNSET,
) -> Optional[Union[CreateUserResponse, HTTPValidationError]]:
    """Create Or Verify User Social

     Create a user using a social login provider.

    All social users are created with `email_is_verified=True`, don't need to be invited and are by
    default read-only
    (unless they are the first user, in which case they are set to admin).

    Args:
        signup_token (Union[None, Unset, str]):
        body (SocialLoginRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreateUserResponse, HTTPValidationError]
    """

    return sync_detailed(client=client, body=body, signup_token=signup_token).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    body: SocialLoginRequest,
    signup_token: Union[None, Unset, str] = UNSET,
) -> Response[Union[CreateUserResponse, HTTPValidationError]]:
    """Create Or Verify User Social

     Create a user using a social login provider.

    All social users are created with `email_is_verified=True`, don't need to be invited and are by
    default read-only
    (unless they are the first user, in which case they are set to admin).

    Args:
        signup_token (Union[None, Unset, str]):
        body (SocialLoginRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[CreateUserResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(body=body, signup_token=signup_token)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    body: SocialLoginRequest,
    signup_token: Union[None, Unset, str] = UNSET,
) -> Optional[Union[CreateUserResponse, HTTPValidationError]]:
    """Create Or Verify User Social

     Create a user using a social login provider.

    All social users are created with `email_is_verified=True`, don't need to be invited and are by
    default read-only
    (unless they are the first user, in which case they are set to admin).

    Args:
        signup_token (Union[None, Unset, str]):
        body (SocialLoginRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[CreateUserResponse, HTTPValidationError]
    """

    return (await asyncio_detailed(client=client, body=body, signup_token=signup_token)).parsed



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.generic_action import GenericAction
from ..models.resource_kind import ResourceKind
from ..types import UNSET, Unset

T = TypeVar("T", bound="GenericPermission")


@_attrs_define
class GenericPermission:
    """Generic permissions describe what a user GENERALLY can and cannot do.

    The actions are not specific to a resource instance (e.g. create). More fine-grained permissions should also be set
    in the authorization policies.

        Attributes:
            action (GenericAction):
            allowed (bool):
            resource (ResourceKind):
            message (Union[None, Unset, str]):
    """

    action: GenericAction
    allowed: bool
    resource: ResourceKind
    message: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action.value

        allowed = self.allowed

        resource = self.resource.value

        message: Union[None, Unset, str]
        if isinstance(self.message, Unset):
            message = UNSET
        else:
            message = self.message

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"action": action, "allowed": allowed, "resource": resource})
        if message is not UNSET:
            field_dict["message"] = message

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        action = GenericAction(d.pop("action"))

        allowed = d.pop("allowed")

        resource = ResourceKind(d.pop("resource"))

        def _parse_message(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        message = _parse_message(d.pop("message", UNSET))

        generic_permission = cls(action=action, allowed=allowed, resource=resource, message=message)

        generic_permission.additional_properties = d
        return generic_permission

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ResourceKind(str, Enum):
    API_KEY = "api_key"
    DATASET = "dataset"
    GENERATED_SCORER = "generated_scorer"
    GROUP = "group"
    GROUP_MEMBER = "group_member"
    INTEGRATION = "integration"
    ORGANIZATION = "organization"
    PROJECT = "project"
    REGISTERED_SCORER = "registered_scorer"
    STAGE = "stage"
    STAGE_VERSION = "stage_version"
    USER = "user"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PromptChainIngestBatchResponse")


@_attrs_define
class PromptChainIngestBatchResponse:
    """
    Attributes:
        job_id (str):
        link (str):
        message (str):
        num_chains (int):
        num_rows (int):
        total_chains (int):
    """

    job_id: str
    link: str
    message: str
    num_chains: int
    num_rows: int
    total_chains: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        job_id = self.job_id

        link = self.link

        message = self.message

        num_chains = self.num_chains

        num_rows = self.num_rows

        total_chains = self.total_chains

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "job_id": job_id,
                "link": link,
                "message": message,
                "num_chains": num_chains,
                "num_rows": num_rows,
                "total_chains": total_chains,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        job_id = d.pop("job_id")

        link = d.pop("link")

        message = d.pop("message")

        num_chains = d.pop("num_chains")

        num_rows = d.pop("num_rows")

        total_chains = d.pop("total_chains")

        prompt_chain_ingest_batch_response = cls(
            job_id=job_id,
            link=link,
            message=message,
            num_chains=num_chains,
            num_rows=num_rows,
            total_chains=total_chains,
        )

        prompt_chain_ingest_batch_response.additional_properties = d
        return prompt_chain_ingest_batch_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.node_type import NodeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.transaction_record_ingest_output_logprobs_type_0 import TransactionRecordIngestOutputLogprobsType0
    from ..models.transaction_record_ingest_user_metadata_type_0 import TransactionRecordIngestUserMetadataType0


T = TypeVar("T", bound="TransactionRecordIngest")


@_attrs_define
class TransactionRecordIngest:
    """
    Attributes:
        created_at (datetime.datetime):
        input_text (str):
        node_id (str):
        node_type (NodeType):
        chain_id (Union[None, UUID, Unset]):
        chain_root_id (Union[None, Unset, str]):
        finish_reason (Union[None, Unset, str]):
        has_children (Union[None, Unset, bool]):
        latency_ms (Union[None, Unset, int]):
        model (Union[None, Unset, str]):
        node_name (Union[None, Unset, str]):
        num_input_tokens (Union[None, Unset, int]):
        num_output_tokens (Union[None, Unset, int]):
        num_total_tokens (Union[None, Unset, int]):
        output_logprobs (Union['TransactionRecordIngestOutputLogprobsType0', None, Unset]):
        output_text (Union[None, Unset, str]):
        status_code (Union[None, Unset, int]):
        tags (Union[None, Unset, list[str]]):
        temperature (Union[None, Unset, float]):
        time_to_first_token_ms (Union[None, Unset, float]):
        tools (Union[None, Unset, str]):
        user_metadata (Union['TransactionRecordIngestUserMetadataType0', None, Unset]):
        version (Union[None, Unset, str]):
    """

    created_at: datetime.datetime
    input_text: str
    node_id: str
    node_type: NodeType
    chain_id: Union[None, UUID, Unset] = UNSET
    chain_root_id: Union[None, Unset, str] = UNSET
    finish_reason: Union[None, Unset, str] = UNSET
    has_children: Union[None, Unset, bool] = UNSET
    latency_ms: Union[None, Unset, int] = UNSET
    model: Union[None, Unset, str] = UNSET
    node_name: Union[None, Unset, str] = UNSET
    num_input_tokens: Union[None, Unset, int] = UNSET
    num_output_tokens: Union[None, Unset, int] = UNSET
    num_total_tokens: Union[None, Unset, int] = UNSET
    output_logprobs: Union["TransactionRecordIngestOutputLogprobsType0", None, Unset] = UNSET
    output_text: Union[None, Unset, str] = UNSET
    status_code: Union[None, Unset, int] = UNSET
    tags: Union[None, Unset, list[str]] = UNSET
    temperature: Union[None, Unset, float] = UNSET
    time_to_first_token_ms: Union[None, Unset, float] = UNSET
    tools: Union[None, Unset, str] = UNSET
    user_metadata: Union["TransactionRecordIngestUserMetadataType0", None, Unset] = UNSET
    version: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.transaction_record_ingest_output_logprobs_type_0 import TransactionRecordIngestOutputLogprobsType0
        from ..models.transaction_record_ingest_user_metadata_type_0 import TransactionRecordIngestUserMetadataType0

        created_at = self.created_at.isoformat()

        input_text = self.input_text

        node_id = self.node_id

        node_type = self.node_type.value

        chain_id: Union[None, Unset, str]
        if isinstance(self.chain_id, Unset):
            chain_id = UNSET
        elif isinstance(self.chain_id, UUID):
            chain_id = str(self.chain_id)
        else:
            chain_id = self.chain_id

        chain_root_id: Union[None, Unset, str]
        if isinstance(self.chain_root_id, Unset):
            chain_root_id = UNSET
        else:
            chain_root_id = self.chain_root_id

        finish_reason: Union[None, Unset, str]
        if isinstance(self.finish_reason, Unset):
            finish_reason = UNSET
        else:
            finish_reason = self.finish_reason

        has_children: Union[None, Unset, bool]
        if isinstance(self.has_children, Unset):
            has_children = UNSET
        else:
            has_children = self.has_children

        latency_ms: Union[None, Unset, int]
        if isinstance(self.latency_ms, Unset):
            latency_ms = UNSET
        else:
            latency_ms = self.latency_ms

        model: Union[None, Unset, str]
        if isinstance(self.model, Unset):
            model = UNSET
        else:
            model = self.model

        node_name: Union[None, Unset, str]
        if isinstance(self.node_name, Unset):
            node_name = UNSET
        else:
            node_name = self.node_name

        num_input_tokens: Union[None, Unset, int]
        if isinstance(self.num_input_tokens, Unset):
            num_input_tokens = UNSET
        else:
            num_input_tokens = self.num_input_tokens

        num_output_tokens: Union[None, Unset, int]
        if isinstance(self.num_output_tokens, Unset):
            num_output_tokens = UNSET
        else:
            num_output_tokens = self.num_output_tokens

        num_total_tokens: Union[None, Unset, int]
        if isinstance(self.num_total_tokens, Unset):
            num_total_tokens = UNSET
        else:
            num_total_tokens = self.num_total_tokens

        output_logprobs: Union[None, Unset, dict[str, Any]]
        if isinstance(self.output_logprobs, Unset):
            output_logprobs = UNSET
        elif isinstance(self.output_logprobs, TransactionRecordIngestOutputLogprobsType0):
            output_logprobs = self.output_logprobs.to_dict()
        else:
            output_logprobs = self.output_logprobs

        output_text: Union[None, Unset, str]
        if isinstance(self.output_text, Unset):
            output_text = UNSET
        else:
            output_text = self.output_text

        status_code: Union[None, Unset, int]
        if isinstance(self.status_code, Unset):
            status_code = UNSET
        else:
            status_code = self.status_code

        tags: Union[None, Unset, list[str]]
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        temperature: Union[None, Unset, float]
        if isinstance(self.temperature, Unset):
            temperature = UNSET
        else:
            temperature = self.temperature

        time_to_first_token_ms: Union[None, Unset, float]
        if isinstance(self.time_to_first_token_ms, Unset):
            time_to_first_token_ms = UNSET
        else:
            time_to_first_token_ms = self.time_to_first_token_ms

        tools: Union[None, Unset, str]
        if isinstance(self.tools, Unset):
            tools = UNSET
        else:
            tools = self.tools

        user_metadata: Union[None, Unset, dict[str, Any]]
        if isinstance(self.user_metadata, Unset):
            user_metadata = UNSET
        elif isinstance(self.user_metadata, TransactionRecordIngestUserMetadataType0):
            user_metadata = self.user_metadata.to_dict()
        else:
            user_metadata = self.user_metadata

        version: Union[None, Unset, str]
        if isinstance(self.version, Unset):
            version = UNSET
        else:
            version = self.version

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {"created_at": created_at, "input_text": input_text, "node_id": node_id, "node_type": node_type}
        )
        if chain_id is not UNSET:
            field_dict["chain_id"] = chain_id
        if chain_root_id is not UNSET:
            field_dict["chain_root_id"] = chain_root_id
        if finish_reason is not UNSET:
            field_dict["finish_reason"] = finish_reason
        if has_children is not UNSET:
            field_dict["has_children"] = has_children
        if latency_ms is not UNSET:
            field_dict["latency_ms"] = latency_ms
        if model is not UNSET:
            field_dict["model"] = model
        if node_name is not UNSET:
            field_dict["node_name"] = node_name
        if num_input_tokens is not UNSET:
            field_dict["num_input_tokens"] = num_input_tokens
        if num_output_tokens is not UNSET:
            field_dict["num_output_tokens"] = num_output_tokens
        if num_total_tokens is not UNSET:
            field_dict["num_total_tokens"] = num_total_tokens
        if output_logprobs is not UNSET:
            field_dict["output_logprobs"] = output_logprobs
        if output_text is not UNSET:
            field_dict["output_text"] = output_text
        if status_code is not UNSET:
            field_dict["status_code"] = status_code
        if tags is not UNSET:
            field_dict["tags"] = tags
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if time_to_first_token_ms is not UNSET:
            field_dict["time_to_first_token_ms"] = time_to_first_token_ms
        if tools is not UNSET:
            field_dict["tools"] = tools
        if user_metadata is not UNSET:
            field_dict["user_metadata"] = user_metadata
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.transaction_record_ingest_output_logprobs_type_0 import TransactionRecordIngestOutputLogprobsType0
        from ..models.transaction_record_ingest_user_metadata_type_0 import TransactionRecordIngestUserMetadataType0

        d = src_dict.copy()
        created_at = isoparse(d.pop("created_at"))

        input_text = d.pop("input_text")

        node_id = d.pop("node_id")

        node_type = NodeType(d.pop("node_type"))

        def _parse_chain_id(data: object) -> Union[None, UUID, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                chain_id_type_0 = UUID(data)

                return chain_id_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, UUID, Unset], data)

        chain_id = _parse_chain_id(d.pop("chain_id", UNSET))

        def _parse_chain_root_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        chain_root_id = _parse_chain_root_id(d.pop("chain_root_id", UNSET))

        def _parse_finish_reason(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        finish_reason = _parse_finish_reason(d.pop("finish_reason", UNSET))

        def _parse_has_children(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        has_children = _parse_has_children(d.pop("has_children", UNSET))

        def _parse_latency_ms(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        latency_ms = _parse_latency_ms(d.pop("latency_ms", UNSET))

        def _parse_model(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        model = _parse_model(d.pop("model", UNSET))

        def _parse_node_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        node_name = _parse_node_name(d.pop("node_name", UNSET))

        def _parse_num_input_tokens(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_input_tokens = _parse_num_input_tokens(d.pop("num_input_tokens", UNSET))

        def _parse_num_output_tokens(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_output_tokens = _parse_num_output_tokens(d.pop("num_output_tokens", UNSET))

        def _parse_num_total_tokens(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_total_tokens = _parse_num_total_tokens(d.pop("num_total_tokens", UNSET))

        def _parse_output_logprobs(data: object) -> Union["TransactionRecordIngestOutputLogprobsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_logprobs_type_0 = TransactionRecordIngestOutputLogprobsType0.from_dict(data)

                return output_logprobs_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TransactionRecordIngestOutputLogprobsType0", None, Unset], data)

        output_logprobs = _parse_output_logprobs(d.pop("output_logprobs", UNSET))

        def _parse_output_text(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        output_text = _parse_output_text(d.pop("output_text", UNSET))

        def _parse_status_code(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        status_code = _parse_status_code(d.pop("status_code", UNSET))

        def _parse_tags(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tags_type_0 = cast(list[str], data)

                return tags_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_temperature(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        temperature = _parse_temperature(d.pop("temperature", UNSET))

        def _parse_time_to_first_token_ms(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        time_to_first_token_ms = _parse_time_to_first_token_ms(d.pop("time_to_first_token_ms", UNSET))

        def _parse_tools(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        tools = _parse_tools(d.pop("tools", UNSET))

        def _parse_user_metadata(data: object) -> Union["TransactionRecordIngestUserMetadataType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                user_metadata_type_0 = TransactionRecordIngestUserMetadataType0.from_dict(data)

                return user_metadata_type_0
            except:  # noqa: E722
                pass
            return cast(Union["TransactionRecordIngestUserMetadataType0", None, Unset], data)

        user_metadata = _parse_user_metadata(d.pop("user_metadata", UNSET))

        def _parse_version(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        version = _parse_version(d.pop("version", UNSET))

        transaction_record_ingest = cls(
            created_at=created_at,
            input_text=input_text,
            node_id=node_id,
            node_type=node_type,
            chain_id=chain_id,
            chain_root_id=chain_root_id,
            finish_reason=finish_reason,
            has_children=has_children,
            latency_ms=latency_ms,
            model=model,
            node_name=node_name,
            num_input_tokens=num_input_tokens,
            num_output_tokens=num_output_tokens,
            num_total_tokens=num_total_tokens,
            output_logprobs=output_logprobs,
            output_text=output_text,
            status_code=status_code,
            tags=tags,
            temperature=temperature,
            time_to_first_token_ms=time_to_first_token_ms,
            tools=tools,
            user_metadata=user_metadata,
            version=version,
        )

        transaction_record_ingest.additional_properties = d
        return transaction_record_ingest

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TrainJobStatusResponse")


@_attrs_define
class TrainJobStatusResponse:
    """
    Attributes:
        history (list[Any]):
        id (str):
        status (str):
        status_message (str):
        project_id (Union[None, Unset, str]):
        project_name (Union[None, Unset, str]):
        run_id (Union[None, Unset, str]):
        run_name (Union[None, Unset, str]):
    """

    history: list[Any]
    id: str
    status: str
    status_message: str
    project_id: Union[None, Unset, str] = UNSET
    project_name: Union[None, Unset, str] = UNSET
    run_id: Union[None, Unset, str] = UNSET
    run_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        history = self.history

        id = self.id

        status = self.status

        status_message = self.status_message

        project_id: Union[None, Unset, str]
        if isinstance(self.project_id, Unset):
            project_id = UNSET
        else:
            project_id = self.project_id

        project_name: Union[None, Unset, str]
        if isinstance(self.project_name, Unset):
            project_name = UNSET
        else:
            project_name = self.project_name

        run_id: Union[None, Unset, str]
        if isinstance(self.run_id, Unset):
            run_id = UNSET
        else:
            run_id = self.run_id

        run_name: Union[None, Unset, str]
        if isinstance(self.run_name, Unset):
            run_name = UNSET
        else:
            run_name = self.run_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"history": history, "id": id, "status": status, "status_message": status_message})
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if project_name is not UNSET:
            field_dict["project_name"] = project_name
        if run_id is not UNSET:
            field_dict["run_id"] = run_id
        if run_name is not UNSET:
            field_dict["run_name"] = run_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        history = cast(list[Any], d.pop("history"))

        id = d.pop("id")

        status = d.pop("status")

        status_message = d.pop("status_message")

        def _parse_project_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project_id = _parse_project_id(d.pop("project_id", UNSET))

        def _parse_project_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        project_name = _parse_project_name(d.pop("project_name", UNSET))

        def _parse_run_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        run_id = _parse_run_id(d.pop("run_id", UNSET))

        def _parse_run_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        run_name = _parse_run_name(d.pop("run_name", UNSET))

        train_job_status_response = cls(
            history=history,
            id=id,
            status=status,
            status_message=status_message,
            project_id=project_id,
            project_name=project_name,
            run_id=run_id,
            run_name=run_name,
        )

        train_job_status_response.additional_properties = d
        return train_job_status_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alerts_configuration import AlertsConfiguration
    from ..models.scorers_config import ScorersConfig


T = TypeVar("T", bound="GenAIProjectSettings")


@_attrs_define
class GenAIProjectSettings:
    """
    Attributes:
        alerts_configuration (Union['AlertsConfiguration', None, Unset]):
        scorers_config (Union['ScorersConfig', None, Unset]):
    """

    alerts_configuration: Union["AlertsConfiguration", None, Unset] = UNSET
    scorers_config: Union["ScorersConfig", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.alerts_configuration import AlertsConfiguration
        from ..models.scorers_config import ScorersConfig

        alerts_configuration: Union[None, Unset, dict[str, Any]]
        if isinstance(self.alerts_configuration, Unset):
            alerts_configuration = UNSET
        elif isinstance(self.alerts_configuration, AlertsConfiguration):
            alerts_configuration = self.alerts_configuration.to_dict()
        else:
            alerts_configuration = self.alerts_configuration

        scorers_config: Union[None, Unset, dict[str, Any]]
        if isinstance(self.scorers_config, Unset):
            scorers_config = UNSET
        elif isinstance(self.scorers_config, ScorersConfig):
            scorers_config = self.scorers_config.to_dict()
        else:
            scorers_config = self.scorers_config

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if alerts_configuration is not UNSET:
            field_dict["alerts_configuration"] = alerts_configuration
        if scorers_config is not UNSET:
            field_dict["scorers_config"] = scorers_config

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.alerts_configuration import AlertsConfiguration
        from ..models.scorers_config import ScorersConfig

        d = src_dict.copy()

        def _parse_alerts_configuration(data: object) -> Union["AlertsConfiguration", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                alerts_configuration_type_0 = AlertsConfiguration.from_dict(data)

                return alerts_configuration_type_0
            except:  # noqa: E722
                pass
            return cast(Union["AlertsConfiguration", None, Unset], data)

        alerts_configuration = _parse_alerts_configuration(d.pop("alerts_configuration", UNSET))

        def _parse_scorers_config(data: object) -> Union["ScorersConfig", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                scorers_config_type_0 = ScorersConfig.from_dict(data)

                return scorers_config_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ScorersConfig", None, Unset], data)

        scorers_config = _parse_scorers_config(d.pop("scorers_config", UNSET))

        gen_ai_project_settings = cls(alerts_configuration=alerts_configuration, scorers_config=scorers_config)

        gen_ai_project_settings.additional_properties = d
        return gen_ai_project_settings

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
import logging
from typing import Optional, Union, overload

from galileo.base import BaseClientModel
from galileo.resources.api.projects import (
    create_project_projects_post,
    get_all_projects_projects_all_get,
    get_project_projects_project_id_get,
    get_projects_projects_get,
)
from galileo.resources.models.http_validation_error import HTTPValidationError
from galileo.resources.models.permission import Permission
from galileo.resources.models.project_create import ProjectCreate
from galileo.resources.models.project_create_response import ProjectCreateResponse
from galileo.resources.models.project_db import ProjectDB
from galileo.resources.models.project_db_thin import ProjectDBThin
from galileo.resources.models.project_type import ProjectType
from galileo.resources.types import UNSET, Unset
from galileo.utils.catch_log import DecorateAllMethods

_logger = logging.getLogger(__name__)


class Project:
    """
    Represents a project in the Galileo platform.

    Projects are containers for logs, traces, and other data in Galileo. All logs are stored
    within a project, and users can create and manage projects to organize their LLM usage data.

    Attributes:
        created_at (datetime.datetime): The timestamp when the project was created.
        created_by (str): The identifier of the user who created the project.
        id (str): The unique identifier of the project.
        updated_at (datetime.datetime): The timestamp when the project was last updated.
        bookmark (Union[Unset, bool]): Whether the project is bookmarked. Defaults to False.
        name (Union[None, Unset, str]): The name of the project.
        permissions (Union[Unset, list["Permission"]]): The permissions associated with the project.
        type (Union[None, ProjectType, Unset]): The type of the project, typically GEN_AI.

    Examples:
         from galileo.projects import get_project, create_project, list_projects

         # Create a new project
         project = create_project(name="My AI Project")

         # Get a project by name
         project = get_project(name="My AI Project")

         # Get a project by ID
         project = get_project(id="project-id-123")

         # List all projects
         projects = list_projects()
         for project in projects:
             print(f"Project: {project.name} (ID: {project.id})")
    """

    created_at: datetime.datetime
    created_by: str
    id: str
    updated_at: datetime.datetime
    bookmark: Union[Unset, bool] = False
    name: Union[None, Unset, str] = UNSET
    permissions: Union[Unset, list["Permission"]] = UNSET
    type: Union[None, ProjectType, Unset] = UNSET

    def __init__(self, project: Union[None, ProjectDBThin, ProjectDB, ProjectCreateResponse] = None) -> None:
        """
        Initialize a Project instance.

        Args:
            project (Union[None, ProjectDBThin, ProjectDB, ProjectCreateResponse], optional):
                The project data to initialize from. If None, creates an empty project instance.
                Defaults to None.
        """
        if project is None:
            return

        self.created_at = project.created_at
        self.created_by = project.created_by
        self.id = project.id
        self.updated_at = project.updated_at
        self.name = project.name
        self.type = project.type_

        if isinstance(project, ProjectDBThin) or isinstance(project, ProjectDB):
            self.bookmark = project.bookmark
            self.permissions = project.permissions


class Projects(BaseClientModel, DecorateAllMethods):
    def list(self) -> list[Project]:
        """
        Lists all projects.

        Returns
        -------
        list[Project]
            A list of projects.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        projects: list[ProjectDBThin] = get_all_projects_projects_all_get.sync(
            client=self.client, type_=ProjectType.GEN_AI
        )
        return [Project(project=project) for project in projects] if projects else []

    @overload
    def get(self, *, id: str) -> Optional[Project]: ...
    @overload
    def get(self, *, name: str) -> Optional[Project]: ...
    def get(self, *, id: Optional[str] = None, name: Optional[str] = None) -> Optional[Project]:
        """
        Retrieves a project by id or name (exactly one of `id` or `name` must be provided).

        Parameters
        ----------
        id : str
            The id of the project.
        name : str
            The name of the project.
        Returns
        -------
        Project
            The project.

        Raises
        ------
        ValueError
            If neither or both `id` and `name` are provided.
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        if (id is None) and (name is None):
            raise ValueError("Exactly one of 'id' or 'name' must be provided")

        if id:
            project_response = get_project_projects_project_id_get.sync(project_id=id, client=self.client)
            if not project_response:
                return None
            project = Project(project=project_response)

        elif name:
            projects_response = get_projects_projects_get.sync(
                client=self.client, project_name=name, type_=ProjectType.GEN_AI
            )

            if not projects_response or len(projects_response) == 0:
                return None

            project = Project(project=projects_response[0])

        return project

    def create(self, name: str) -> Project:
        """
        Creates a new project.

        Parameters
        ----------
        name : str
            The name of the project.

        Returns
        -------
        Project
            The created project.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        body = ProjectCreate(name=name, type_=ProjectType.GEN_AI, create_example_templates=False, created_by=None)

        response = create_project_projects_post.sync(client=self.client, body=body)

        if isinstance(response, HTTPValidationError):
            _logger.error(response)
            raise response

        if not response:
            raise ValueError(f"Unable to create project: {name}")

        return Project(project=response)


#
# Convenience methods
#


def get_project(*, id: Optional[str] = None, name: Optional[str] = None) -> Optional[Project]:
    """
    Retrieves a project by id or name (exactly one of `id` or `name` must be provided).

    Parameters
    ----------
    id : str
        The id of the project.
    name : str
        The name of the project.
    with_content : bool
        Whether to return the content of the project. Default is False.

    Returns
    -------
    Project
        The project.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Projects().get(id=id, name=name)


def list_projects() -> list[Project]:
    """
    Lists all projects.

    Parameters
    ----------
    limit : Union[Unset, int]
        The maximum number of projects to return. Default is 100.

    Returns
    -------
    list[Project]
        A list of projects.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Projects().list()


def create_project(name: str) -> Project:
    """
    Creates a new project.

    Parameters
    ----------
    name : str
        The name of the project.
    type_ : ProjectType
        The type of the project.

    Returns
    -------
    Project
        The created project.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Projects().create(name=name)



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.group_visibility import GroupVisibility

T = TypeVar("T", bound="GroupUpdate")


@_attrs_define
class GroupUpdate:
    """
    Attributes:
        description (Union[None, str]):
        name (str):
        visibility (GroupVisibility):
    """

    description: Union[None, str]
    name: str
    visibility: GroupVisibility
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        description: Union[None, str]
        description = self.description

        name = self.name

        visibility = self.visibility.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"description": description, "name": name, "visibility": visibility})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_description(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        description = _parse_description(d.pop("description"))

        name = d.pop("name")

        visibility = GroupVisibility(d.pop("visibility"))

        group_update = cls(description=description, name=name, visibility=visibility)

        group_update.additional_properties = d
        return group_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.bar_chart import BarChart


T = TypeVar("T", bound="SegmentationMetricsResponse")


@_attrs_define
class SegmentationMetricsResponse:
    """The main metric for Semantic Segmentation.

    IoU, or Intersection over Union, is an Semantic Segmentation metric that provides you with a general sense of the
    performance of your model.

    IoU is defined as the intersection of the predicted and ground truth masks Mean IoU is the average IoU across all
    classes Boundary IoU is the IoU for the boundary of the mask, which protects against   bias towards larger masks
    https://learnopencv.com/intersection-over-union-iou-in-object-detection-and-segmentation/

        Attributes:
            boundary_iou (float):
            boundary_iou_per_class (BarChart): A class to represent a basic bar chart.

                labels: List[str] the x axis labels
                values: List[int | float] the counts for each bar
            dice_coefficient (float):
            dice_per_class (BarChart): A class to represent a basic bar chart.

                labels: List[str] the x axis labels
                values: List[int | float] the counts for each bar
            mean_iou (float):
            mean_iou_per_class (BarChart): A class to represent a basic bar chart.

                labels: List[str] the x axis labels
                values: List[int | float] the counts for each bar
    """

    boundary_iou: float
    boundary_iou_per_class: "BarChart"
    dice_coefficient: float
    dice_per_class: "BarChart"
    mean_iou: float
    mean_iou_per_class: "BarChart"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        boundary_iou = self.boundary_iou

        boundary_iou_per_class = self.boundary_iou_per_class.to_dict()

        dice_coefficient = self.dice_coefficient

        dice_per_class = self.dice_per_class.to_dict()

        mean_iou = self.mean_iou

        mean_iou_per_class = self.mean_iou_per_class.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "boundary_iou": boundary_iou,
                "boundary_iou_per_class": boundary_iou_per_class,
                "dice_coefficient": dice_coefficient,
                "dice_per_class": dice_per_class,
                "mean_iou": mean_iou,
                "mean_iou_per_class": mean_iou_per_class,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.bar_chart import BarChart

        d = src_dict.copy()
        boundary_iou = d.pop("boundary_iou")

        boundary_iou_per_class = BarChart.from_dict(d.pop("boundary_iou_per_class"))

        dice_coefficient = d.pop("dice_coefficient")

        dice_per_class = BarChart.from_dict(d.pop("dice_per_class"))

        mean_iou = d.pop("mean_iou")

        mean_iou_per_class = BarChart.from_dict(d.pop("mean_iou_per_class"))

        segmentation_metrics_response = cls(
            boundary_iou=boundary_iou,
            boundary_iou_per_class=boundary_iou_per_class,
            dice_coefficient=dice_coefficient,
            dice_per_class=dice_per_class,
            mean_iou=mean_iou,
            mean_iou_per_class=mean_iou_per_class,
        )

        segmentation_metrics_response.additional_properties = d
        return segmentation_metrics_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Galileo"""

# flake8: noqa: F401
# ruff: noqa: F401

from galileo.decorator import GalileoDecorator, galileo_context, log
from galileo.logger import GalileoLogger
from galileo_core.helpers.api_key import create_api_key, delete_api_key, list_api_keys
from galileo_core.helpers.dependencies import is_dependency_available
from galileo_core.schemas.shared.traces.types import LlmSpan, RetrieverSpan, ToolSpan, Trace, WorkflowSpan
from galileo_core.schemas.shared.workflows.node_type import NodeType
from galileo_core.schemas.shared.workflows.step import (
    AgentStep,
    LlmStep,
    RetrieverStep,
    StepWithChildren,
    ToolStep,
    WorkflowStep,
)

__version__ = "0.2.4"



from enum import Enum


class Routes(str, Enum):
    healthcheck = "healthcheck"
    login = "login"
    api_key_login = "login/api_key"
    get_token = "get-token"

    projects = "projects"
    all_projects = "projects/all"

    log_streams = "/v2/projects/{project_id}/log_streams"

    traces = "/v2/projects/{project_id}/traces"
    traces_search = "/v2/projects/{project_id}/traces/search"
    traces_available_columns = "/v2/projects/{project_id}/traces/available_columns"
    trace = "/v2/projects/{project_id}/traces/{trace_id}"
    spans = "/v2/projects/{project_id}/spans"
    spans_search = "/v2/projects/{project_id}/spans/search"
    spans_available_columns = "/v2/projects/{project_id}/spans/available_columns"
    span = "/v2/projects/{project_id}/spans/{span_id}"



from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Seq2SeqMetricsResponse")


@_attrs_define
class Seq2SeqMetricsResponse:
    """Returns Seq2Seq specific metrics.

    Bleu and Rouge are the most common metrics for Seq2Seq models We also include average data error potential

        Attributes:
            average_bleu (Union[None, Unset, float]):
            average_data_error_potential (Union[None, Unset, float]):
            average_rouge (Union[None, Unset, float]):
    """

    average_bleu: Union[None, Unset, float] = UNSET
    average_data_error_potential: Union[None, Unset, float] = UNSET
    average_rouge: Union[None, Unset, float] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        average_bleu: Union[None, Unset, float]
        if isinstance(self.average_bleu, Unset):
            average_bleu = UNSET
        else:
            average_bleu = self.average_bleu

        average_data_error_potential: Union[None, Unset, float]
        if isinstance(self.average_data_error_potential, Unset):
            average_data_error_potential = UNSET
        else:
            average_data_error_potential = self.average_data_error_potential

        average_rouge: Union[None, Unset, float]
        if isinstance(self.average_rouge, Unset):
            average_rouge = UNSET
        else:
            average_rouge = self.average_rouge

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if average_bleu is not UNSET:
            field_dict["average_bleu"] = average_bleu
        if average_data_error_potential is not UNSET:
            field_dict["average_data_error_potential"] = average_data_error_potential
        if average_rouge is not UNSET:
            field_dict["average_rouge"] = average_rouge

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_average_bleu(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_bleu = _parse_average_bleu(d.pop("average_bleu", UNSET))

        def _parse_average_data_error_potential(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_data_error_potential = _parse_average_data_error_potential(d.pop("average_data_error_potential", UNSET))

        def _parse_average_rouge(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        average_rouge = _parse_average_rouge(d.pop("average_rouge", UNSET))

        seq_2_seq_metrics_response = cls(
            average_bleu=average_bleu,
            average_data_error_potential=average_data_error_potential,
            average_rouge=average_rouge,
        )

        seq_2_seq_metrics_response.additional_properties = d
        return seq_2_seq_metrics_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
